define matrixmul()
%!%+
%\function{matrixmul}
%\synopsis{Matrix multiplication}
%\usage{ M[n,m] = matrixmul( A[n,j], B[j,m] );}
%\description
%    (Efficient) Calculation of the product of two matrices A and B.
%    If A is a (n x j)-Matrix and B a (j x m)-Matrix the resulting
%    Matrix has the dimensions (n x m).
%
%    If A or B is one dimensional, the neccessary second dimension
%    is assumed to be of length 1,
%    i.e., A[n]*B[m] =  A[n,1]*B[1,m] = M[n,m]
%
%    This function is identical to the ISIS intrinsic operator #
%    or the inner product and only available for backwards
%    compatibility!
%
%\seealso{transpose, inner_product}
%!%-
{
  
  variable _a, _b;
  switch(_NARGS)
  { case 2: ( _a, _b ) = (); }
  { help(_function_name); return;}

  if( length(array_shape(_a))==length(array_shape(_b))==1 ){
    return _a # transpose(_b);
  }else{
    return _a # _b;
  }  
}
