private define indarray( s, e ){
  return [s:e];
}
private define __get( a, i ){
  return a[i[where(i<length(a))]];
}

define array2matrix()
%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{array2matrix}
%\synopsis{Transforms an Array_Type into a matrix}
%\usage{Any_Type[l,...] = array2matrix(Array_Type[l] array);}
%\qualifiers{
%\qualifier{filler[=-_NaN]:}{Filler for empty array entries}
%}
%\description
%    Similar to array_flatten, this function flattens the
%    array into a single one. In contrast to array_flatten,
%    the array_shape of the individual entries is conserved.
%    If the array_shape of the individual array entries is
%    not the same, the maximal dimension is determined and
%    empty entries are being filled with the value given
%    with the filler qualifier.
%\example
%    variable arr = Array_Type[3];
%    arr[0] = _reshape( [1:3*4*5], [3,4,5] );
%    arr[1] = _reshape( [1:3*4], [3,4] );
%    arr[2] = _reshape( [1:2*3*4], [2,3,4] );
%    
%    variable mat = array2matrix(arr);
%    print(mat);
%\seealso{array_flatten}
%!%-
{
  variable filler = qualifier("filler",-_NaN);
    
  variable A;
  switch(_NARGS)
  { case 1: A = (); }
  { help(_function_name); return; }

  variable len = length(A); % length of Array
  variable dim = array_map( Array_Type, &array_shape, A ); % dimensions of each Array entry
  variable udim = dim[unique(dim)];
  
  variable M;
  if( length(udim) == 1 ){
    M = array_flatten( A );
    reshape( M, [len,dim[0]] );
  }
  else{
    % Find the maximal length in each dimension
    variable mdlen= max(array_map(Integer_Type, &length, udim)); % #dims for each entry
    variable mdim = Integer_Type[mdlen];
    _for $1 ( 0, mdlen-1 ){ % in each dimension find the maximum
      mdim[$1] = max(array_flatten(array_map( Array_Type, &__get, udim, $1 )));
    }

    variable IND;
    M = (@filler)[Integer_Type[len,__push_array(mdim)]]; % Create matrix with max dimension
    _for $1 ( 0, len-1 ){
      IND = array_map( Array_Type, &indarray, 0, [ dim[$1]-1, Integer_Type[length(mdim)-length(dim[$1])] ] );
      M[ $1, __push_array(IND) ] = A[ $1 ];
    }
  }
  
  return M;
}
