%{{{% array_map_index private functions
private define __aindex (start, stop, arg) {
  variable n = (typeof(start)==Ref_Type)?@start:[start:stop];
  return struct{value=(typeof(arg.value)==Array_Type)?arg.value[n]:arg.value};
}
private define __findex (start, stop, f, args_ref) {
  % single run of array_map_index
  variable args = array_map(Struct_Type, &__aindex, start, stop, @args_ref);
  return @f(__push_args(args));
}
%}}}%

define array_map_index ()
%{{{%
%!%+
%\function{array_map_index}
%\synopsis{Call a function on a subset of arrays}
%\usage{Array_Type array_map_index (Int_Type[] start, Int_Type[] stop,
%                              DataType_Type type, Ref_Type func, args, ...);}
%\altusage{Array_Type array_map_index (Int_Type[] index, DataType_Type type,
%                              Ref_Type func, args, ...);}
%\altusage{Array_Type array_map_index (Ref_Type start, Ref_Type stop,
%                              DataType_Type type, Ref_Type func, args, ...);}
%\altusage{Array_Type array_map_index (Ref_Type index, DataType_Type type,
%                              Ref_Type func, args, ...);}
%\description
%  The \code{array_map_index} function is somewhat the opposite of the
%  \code{array_map}. As \code{array_map} allows it to apply a function to every
%  array element individually, \code{array_map_index} can be used to apply a
%  function (taking an array) on one or more subsamples of the target array.
%
%  Case 1: If called with \code{start} and \code{stop} the function \code{func} is
%  applied to \code{args[start[i]:stop[i]]} for each \code{i}.
%
%  Case 2: If called with only one index array \code{index} the function is
%  applied to args[index]. This is the same as \code{func(args[index]);}.
%
%  IMPORTANT: array_map_index applys \code{func} to an array!
%
%  Case 3: If called with \code{&start} and \code{&stop} the function behaves the
%  same as for case 1, but this is usefull when you want to nest \code{array_map}
%  calls.
%
%  Case 4: If called with \code{index} the it behaves the same as for case 2 but,
%  again, this might be usefull for \code{array_map} nesting.
%
%  NOTE: This function can hide a lot of important operations so it should be
%  avoided for implementing algorithms. However, it is very useful for fast
%  scripting and interactive (shell) operations.
%
%\example
%  % Case 1: apply mean() to a subset of data
%  data = rand_gauss(0.1, 100) % generate 100 normal distributed random numbers
%  means = array_map_index([0,5,25], [99,94,74], &mean, data);
%  print(means);
%  % this outputs the same as
%  print(mean(data)); print(mean(data[[5:94]])); print(mean(data[[25:74]]));
%
%  % Case 2: apply mean() to specified subset
%  data = rand_gauss(0.1, 100) % 100 random numbers
%  idx = where(data < 0) % select only negative data
%  neg_mean = array_map_index(idx, Double_Type, &mean, data);
%  print(neg_mean);
%  % this results in the same as
%  print(mean(data[n]));
%
%  % Case 3: apply mean() to array of arrays
%  data = Array_Type[3];
%  data[0] = rand_gauss(0.1, 100);
%  data[1] = rand_gauss(0.01, 100);
%  data[2] = rand_gauss(1, 100);
%  start = [0,5,25];
%  stop = [99,94,74];
%  means = array_map(Array_Type, &array_map_index, &start, &stop,
%                    Double_Type, &mean, data);
%  % returns the mean as in example for case 1 but applied to all sub arrays
%  % note that you have to declare the start and stop arrays explicitly because
%  % &[5:94] is not a valid expression
%
%  % Case 4: Same as case 3 but with only one index array
%
%  % Use array_map_index to call a function n times
%  rn = array_map_index([1:5], Array_Type, &rand_gauss, 0.1, 100);
%  % gives 5 arrays of 100 random numbers each
%
%\seealso{array_map, array_map_qualifiers}
%!%-
{
  variable rt, start, stop, f;
  variable args = __pop_args(_NARGS);
  (start, stop, rt) = __push_args(args[[:2]]);
  if (typeof(stop) == DataType_Type) {
    if (typeof(start) == Ref_Type)
      return array_map(stop, &__findex, start, @start, rt, &(args[[3:]])); % stop is $
    else
      return array_map(stop, &__findex, &start, @start, rt, &(args[[3:]])); % stop is$
  } else {
    f = __push_args(args[3]);
    if (typeof(start) == Ref_Type)
      return array_map(rt, &__findex, @start, @stop, f, &(args[[4:]]));
    else
      return array_map(rt, &__findex, start, stop, f, &(args[[4:]]));
  }
}
%}}}%
