%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
define associate_arrays(array1, array2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{associate_arrays}
%\synopsis{computes the smallest pairwise difference between elements of two arrays and returns the indices}
%\usage{Struct_Type result = associate_arrays(Double_Type[n1] A, Double_Type[n2] B);}
%\description
%    For each element \code{A[i]} this function searches the element \code{B[j]}
%    with the smallest difference to \code{A[i]}. It returns the indices \code{j}
%    as \code{result.index[i]} and the corresponding difference as \code{result.diff[i]}.
%\qualifiers{
%\qualifier{nr_closest}{ [=1]: set to N, to obtain the N closest elements.
%                       In this case additionally the index of the second, ..., N-th closest element is
%                       returned as \code{result.index2[*,0], ..., result.index2[*,N-2]},
%                       the same holds for \code{result.diff2}.}
%}
%\seealso{array_sort}
%!%-
{
  variable nc = qualifier("nr_closest", 1);

  variable result = struct {index, diff};
  variable i,j,d,k;
  result.index =  Integer_Type [length(array1), nc];
  result.diff  =  Double_Type  [length(array1), nc];
  result.diff[*,*] = DOUBLE_MAX;

  for (i=0; i<length(array1); i++)
  {
    for (j=0; j<length(array2); j++)
    {
      d = array1[i] - array2[j];

      k = nc-1;
      while ( k >= 0 && abs(d) < abs(result.diff[i,k]) )
        {
          if (k < nc-1)
          {
            result.diff[i,k+1] = result.diff[i,k];
            result.index[i,k+1] = result.index[i,k];
          }
          result.diff[i,k] = d;
          result.index[i,k] = j;
          k--;
        }
    }
  }
  if (nc > 1)
  {
    result = struct_combine(result,"index2","diff2");
    result.index2 = result.index[*,[1:]];
    result.diff2 = result.diff[*,[1:]];
  }
  result.index = result.index[*,0];
  result.diff = result.diff[*,0];
  return result;
}
