%%%%%%%%%%%%%%%%%
define inner_product( A, B )
%%%%%%%%%%%%%%%%%
%!%+
%\function{inner_product}
%\synopsis{}
%\usage{ M[n,...,j] = inner_product( A[n,...,m], B[m,...,j] );}
%\description
%    In ISIS the operator # calculates the inner product of two arrays
%    A, B, i.e., contracts the last and first dimension, respectively.
%    A and B can have an arbitrary number of dimensions, it is just required
%    that the last dimension of A and first dimension of B is of the same
%    size!
%\example
%    A = _reshape( [1:2*3], [2,3] );
%    B = _reshape( [1:3*4], [3,4] );
%    M = inner_product( A, B );
%    N = A # B;
%    vmessage("M\n"); print(M);
%    vmessage("N\n"); print(N);
%\seealso{operator #}
%!%-
{
  return A # B;
}

%%%%%%%%%%%%%%%%%
define inner_products()
%%%%%%%%%%%%%%%%%
%!%+
%\function{inner_products}
%\synopsis{}
%\usage{ M[n,...,l] = inner_products( A[n,...,m], B[m,...,j] [, ..., Z[k,...,l] ] );}
%\description
%    In ISIS the operator # calculates the inner product of two or more arrays.
%    This function is basically nesting the inner_product function!
%\seealso{inner_product, operator #}
%!%-
{
  ifnot( _NARGS ){ help(_function_name); return; }
  
  variable args = __pop_args(_NARGS);
  
  variable IP = __push_args( args[0] );
  variable i;
  _for i ( 1, _NARGS-1 ){
    IP = inner_product( IP, __push_args( args[i] ) );
  }
  return IP;
}
