define outer_products()
%!%+
%\function{outer_products}
%\synopsis{computes multi-dimensional arrays from one-dimensional ones}
%\usage{(Double_Type P1[], P2[], ...) = outer_products(Double_Type f1[], f2[], ..., fn[]);}
%\description
%    \code{P1[i1, i2, ..., in] = f1[i1];}\n
%    \code{P2[i1, i2, ..., in] = f2[i2];}\n
%    ...\n
%    \code{Pn[i1, i2, ..., in] = fn[i1];}\n
%
%    The return value \code{Pj} is calculated as outer product of \code{x1, x2, ..., xn},
%    where \code{xj} = \code{fj} and
%          \code{xi = [1, 1, ..., 1]} (same length as \code{fi}) for \code{i!=j}.
%\seealso{outer_product}
%!%-
{
  if(_NARGS==0)
  { help(_function_name()); return; }

  variable n = _NARGS;
  variable arg = __pop_args(n);
  variable ones = Array_Type[n];
  variable i, j;
  _for i (0, n-1, 1)
    ones[i] = Integer_Type[length(arg[i].value)] + 1;
  _for i (0, n-1, 1)
  { variable list = {};
    _for j (0, n-1, 1)
      if(i==j)
        list_append(list, arg[j].value);
      else
        list_append(list, ones[j]);
    outer_product( __push_list(list) );  % return value is left on stack
  }
}
