define struct_to_assoc_array()
%!%+
%\function{struct_to_assoc_array}
%\synopsis{converts a structure to an associative array}
%\usage{Assoc_Type struct_to_assoc_array(Struct_Type structure[, DataType_Type data-type]);}
%\description
%    The given structure is converted into an associative
%    array. Thereby, the keys of the returned array will
%    be equal to the field names of the structure.
%
%    If the second argument is provided, the values of the
%    associative array will be of that type. Otherwise,
%    the type of the first field of the structure will be
%    used. Note, that all fields have to be of the same
%    data-type!
%\example
%    c = struct_to_assoc_array(struct {
%      pi = 3.141592653589793,
%      c = 2.99792458e10,
%      h = 6.62606876e-27
%    });
%    foreach key (c) using ("keys")
%      vmessage("c[%s] = %e", key, c[key]); ;
%\seealso{Assoc_Type, assoc_get_keys, get_struct_field_names}
%!%-
{
  variable s, type;
  switch(_NARGS)
    { case 1: (s) = (); }
    { case 2: (s, type) = (); }
    { help(_function_name()); return; }

  if (typeof(s) != Struct_Type) { vmessage("error (%s): input is not of Struct_Type", _function_name); return; }

  % get names of struct fields and initialize array
  variable fields = get_struct_field_names(s);
  ifnot (__is_initialized(&type)) type = typeof(get_struct_field(s, fields[0]));
  else if (typeof(type) != DataType_Type) { vmessage("error (%s): given type is not of DataType_Type", _function_name); return; }
  variable a = Assoc_Type[type];

  % loop over fields and add values to array
  variable field;
  foreach field (fields) a[field] = get_struct_field(s, field);
  
  return a;
}
