%%%%%%%%%%%%%%%%%%%%%%%%%%%
define Compton_crosssection()
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{Compton_crosssection}
%\usage{Double_Type sigma = Compton_crosssection(Double_Type E);}
%\description
%    \code{E} is the energy in keV.
%    \code{sigma} is the total Klein-Nishina crosssection in cm^2:
%    \code{sigma = 3/4 sigma[Th] * [  (1+y)/y^3 * { 2y*(1+y)/(1+2y) - log(1+2*y) }  +  log(1+2*y)/(2*y)  +  (1+3y)/(1+2y)^2  ]}
%    where \code{y = E/(511 keV)}.
%\seealso{Rybicki & Lightman (1979)}
%!%-
{
  variable E;
  switch(_NARGS)
  { case 1: E = (); }
  { help(_function_name()); return; }

  variable y = E/510.998903;
  variable twoy = 2*y;
  variable onep2y = 1. + twoy;
  variable log1p2y =log1p(twoy);
  return 3/4. * 6.6524586e-25 * ( (1+y)/y^3*( twoy*(1+y)/onep2y - log1p2y ) + log1p2y/twoy - (onep2y+y)/onep2y^2 );
}
