define deg2dms(deg) {
%!%+
%\function{deg2dms}
%\synopsis{Convert a floating point angle in degrees to degrees, minutes, seconds}
%\usage{(d,m,s) = deg2dms(degree)}
%\qualifiers{
%\qualifier{hours}{If set, return the coordinate in hours, minutes, sec (RA)}
%\qualifier{radian}{If set, the angle is in radians, not degrees}
%}
%\description
% This is a convenience routine to convert astronomical coordinates given
% as a floating point number into a number in degrees, minutes, seconds.
%
% For negative angles, the sign of the coordinate is given to the highest 
% non-zero integer.
%
% Use angle2string to generate strings from angles, and generate_iauname
% if you want to build strings to use in source names.
%
%\seealso{dms2deg, hms2deg, angle2string, generate_iauname}
%!%-
    variable sgn;
    sgn=sign(deg);
    variable angle=abs(deg);

    if (qualifier_exists("radian")) {
	angle*=180./PI;
    }

    if (qualifier_exists("hours")) {
	angle=angle/15.;
    }

    variable d=int(angle);
    angle=(angle-d)*60.;
    variable m=int(angle);
    variable s=(angle-m)*60.;

    if (d==0) {
	if (m==0) {
	    s=sgn*s;
	} else {
	    m=sgn*m;
	}
    } else {
	d=sgn*d;
    }

    return (d,m,s);
}
