define color_complex (z)
%!%+
%\function{color_complex}
%\synopsis{Converts a complex number to a RGB color}
%\usage{Integer_Type color = complex2rgb(Complex_Type z);}
%\description
%    arg(\code{z}) determines the hue,
%    |\code{z}| determines the lightness:
%    0<=|\code{z}|<=1 is mapped from black to color,
%    1<=|\code{z}|<inf is mapped from color to white.
%    The return value is a 24-bit RGB value.
%\qualifiers{
%\qualifier{s[=1.0]}{saturation (from 0 to 1) of the color}
%}
%\seealso{hsl2rgb, png_write}
%!%-
{
  variable s = _max(0.0, _min(1.0, qualifier("s", 1.0)));
  variable h = (atan2(Imag(z), Real(z))/2/PI + 1.0) mod 1.0;
  variable l = abs(z)*0.5;
  variable w = where(l>0.5);
  l[w] = 1.0-0.25/l[w];

  return rgb2hex(hsluv2rgb(h,s,l));
}
