define color_desaturate (color)
%!%+
%\function{color_desaturate}
%\synopsis{Weighted color to gray conversion}
%\usage{UInt_Type gray = color_desaturate(UInt_Type[] color);}
%\description
%    Converts a RGB color, hex encoded, into a weighted gray scale using:
%
%    \code{gray} = (0.3 * R) + (0.59 * G) + (0.11 * B)
%    
%    The 8 bit gray value(s) are return as UInt_Type[] in the interval [0,255].
%
%\seealso{rgb2hsl, hex2rgb}
%!%-
{ 
  variable r, g, b;
  (r,g,b) = hex2rgb(color);

  variable gray = (0.3 * r/255.) + (0.59 * g/255.) + (0.11 * b/255.);

  return rgb2hex(r, g, b);
}
