define color_mix (c1,c2,fraction)
%!%+
%\function{color_mix}
%\synopsis{Mix two rgb colors}
%\usage{UInt_Type color = color_mix (UInt_Type c1, UInt_Type c2, Double_Type fraction);}
%\description
%  This function mixes two colors in rgb space. Given the rgb values in the
%  usual encoding in one 24bit integer, the color is mixed according to 
%  new color  = c*fraction + c2*(1-fraction)
%  The operations are analoguous to the color mixing performed by the xcolor
%  package of LaTeX (the operation is similar to the color1!fraction!color2
%  syntax). 
%  The function returns the integer specifying the new rgb values. \code{fraction}
%  can be an array. The colors can also be a hex string.
%!%-
{
  fraction = _max(0., _min(1., fraction));

  variable r1,g1,b1,r2,g2,b2;

  (r1,g1,b1) = hex2rgb(c1);
  (r2,g2,b2) = hex2rgb(c2);

  return rgb2hex((r1*fraction+(1.0-fraction)*r2)/255.,
		 (g1*fraction+(1.0-fraction)*g2)/255.,
		 (b1*fraction+(1.0-fraction)*b2)/255.);
}
