define color_wavelength (lambda)
%!%+
%\function{color_wavelength}
%\synopsis{Computes the color values of visual light}
%\usage{UInt_Type[] color_wavelength (Double_Type[] lambda);}
%\description
%    \code{380 <= lambda <= 780} is the wavelength in nm.
%    The return value is its 24bit RGB color value.
%
%    The conversion is performed after the code by Dan Bruton,
%    see http://www.physics.sfasu.edu/astro/color.html.
%!%-
{
  variable r = Double_Type[length(lambda)];
  variable g = @r;
  variable b = @r;
  variable w, spec;

  w = where(380 <= lambda < 420);
  spec = 0.3 + 0.7*(lambda[w]-380.)/(420-380);
  r[w] = (440.-lambda[w])/(440-380)*spec;
  % g[w] = 0;
  b[w] = spec;

  w = where(420 <= lambda < 440);
  r[w] = (440.-lambda[w])/(440-380);
  % g[w] = 0;
  b[w] = 255;

  w = where(440 <= lambda < 490);
  % r[w] = 0;
  g[w] = (lambda[w]-440.)/(490-440);
  b[w] = 1.0;

  w = where(490 <= lambda < 510);
  % r[w] = 0;
  g[w] = 1.0;
  b[w] = (510.-lambda[w])/(510-490);

  w = where(510 <= lambda < 580);
  r[w] = (lambda[w]-510.)/(580-510);
  g[w] = 1.0;
  % b[w] = 0;

  w = where(580 <= lambda < 645);
  r[w] = 1.0;
  g[w] = (645.-lambda[w])/(645-580);
  % b[w] = 0;

  w= where(645 <= lambda < 700);
  r[w] = 1.0;
  % b[w] = 0;
  % g[w] = 0;

  w = where(700 <= lambda < 780);
  r[w] = 0.3 + 0.7*(780.-lambda[w])/(780-700);
  % g[w] = 0;
  % b[w] = 0;

  return rgb2hex(r, g, b);
}
