private define qphcol (q, p, h6)
{
  if (h6 < 1.0) return p + (q-p)*h6;
  if (h6 < 3.0) return q;
  if (h6 < 4.0) return p + (q-p)*(4.0-h6);
  return p;
}

private define map_hsl(h, s, l)
{
  if (s == 0)
    return l,l,l;

  variable q = l*(1.0+s);
  if (l > 0.5) q = l+s - l*s;

  variable p = 2.0*l - q;
  
  return qphcol(q, p, (h*6.0+2.0) mod 6.0), qphcol(q, p, h*6.0), qphcol(q, p, (h*6.0+4.0) mod 6.0);
}

define hsl2rgb(h, s, l)
%!%+
%\function{hsl2rgb}
%\synopsis{converts a (hue, saturation, lightness) color to (red, green, blue)}
%\usage{Integer_Type rgb = hsl2rgb(Double_Type h, s, l);}
%\description
%    The hue \code{h} is an angle in color-space. Here, \code{h} is between 0 and 1.
%    \code{h} of red is 0, \code{h} of green is 1/3, and \code{h} of blue is 2/3.
%    The lightness \code{l} of black is 0, and \code{l} of white is 1.
%    Like the hue, the saturation \code{0<=s<=1} matters only for \code{0<l<1}.
%
%    The return value is a(n array of) 24 bit RGB value(s).
%\seealso{rgb2hsl}
%!%-
{
  variable r, g, b;

  (r, g, b) = array_map(Double_Type, Double_Type, Double_Type, &map_hsl, h, s, l);

  return length(r) == 1 ? (int(r*255)[0], int(g*255)[0], int(b*255)[0]) :
    (int(r*255), int(g*255), int(b*255));
}

private define map_rhsl (ma, mi, l, r, g, b)
{
  if (ma == mi)
    return 0.0, 0.0, l;

  variable d = ma - mi;
  variable s = l > 0.5 ? d/(2 - ma - mi) : d/(ma+mi);
  variable h;

  switch (ma)
  { case r: h = (g - b)/d + (g < b ? 6.0 : 0.0); }
  { case g: h = (b - r)/d + 2.0; }
  { case b: h = (r - g)/d + 4.0; }

  return h, s, l;
}

define rgb2hsl(r, g, b)
%!%+
%\function{rgb2hsl}
%\synopsis{converts (red, green, blue) values to (hue, saturation, lightness)}
%\usage{(Double_Type h, s, l) = rgb2hsl(Integer_Type r, g, b);}
%\description
%    Converts 8 bit RGB values (0-255) to HSL values.
%
%    The return values are always normalized to [0, 1].
%    See \code{hsl2rgb} for a definition of hue, saturation, lightness.
%\seealso{hsl2rgb}
%!%-
{
  r = r/255.;
  g = g/255.;
  b = b/255.;

  variable ma = _max(r, g, b);
  variable mi = _min(r, g, b);
  variable h, s, l;

  l = (ma+mi)/2.0;

  (h,s,l) = array_map(Double_Type, Double_Type, Double_Type, &map_rhsl, ma, mi, l, r, g, b);

  return length(h) == 1 ? (h[0],s[0],l[0]) : (h, s, l);
}
