private define map_hue (i,v,q,p,t)
{
  switch (i mod 6)
  { case 0: return v, t, p; }
  { case 1: return q, v, p; }
  { case 2: return p, v, t; }
  { case 3: return p, q, v; }
  { case 4: return t, p, v; }
  { case 5: return v, p, q; }
}

define hsv2rgb (h, s, v)
%!%+
%\function{hsv2rgb}
%\synopsis{Convert HSV color to RGB}
%\usage{(Int_Type r,g,b) = hsv2rgb(Double_Type h,s,v);}
%\description
%   HSV (Hue, Saturation, Value) as doubles (0-1) are
%   converted to RGB values (defined as [0-255] 8 bit
%   values). Works for array values two (lengths must
%   match).
%!%-
{
  variable i = int(floor(h*6));
  variable f = h*6-i;
  variable p = v*(1-s);
  variable q = v*(1-f*s);
  variable t = v*(1 - (1 - f)*s);

  variable r,g,b;
  (r,g,b) = array_map(Double_Type,Double_Type,Double_Type,&map_hue,i,v,q,p,t);

  return length(r) == 1 ? (int(r*255)[0], int(g*255)[0], int(b*255)[0]) :
    (int(r*255), int(g*255), int(b*255));
}

private define map_max (ma, mi, r, g, b, d)
{
  variable h;
  if (ma == mi)
    return 0.0;
  else
  {
    switch (ma)
    { case r: h = (g-b)/d + (g < b ? 6.0 : 0.0); }
    { case g: h = (b-r)/d + 2.0; }
    { case b: h = (r-g)/d + 4.0; }
  }

  return h/6.0;
}

define rgb2hsv (r, g, b)
%!%+
%\function{rgb2hsv}
%\synopsis{Convert RGB color to HSV}
%\usage{(Double_Type h,s,v) = rgb2hsv(Int_Type r,g,b);}
%\description
%   RGB colores encoded as 8 bit (0-255) values get
%   converted to HSV (Hue, Saturation, Value) as doubles
%   (0-1) are. Works for array values two (lengths must
%   match).
%!%-
{
  r = r/255.;
  g = g/255.;
  b = b/255.;

  variable ma = _max(r, g, b);
  variable mi = _min(r, g, b);
  variable h, s, v;

  v = ma;

  variable d = ma - mi;

  s = ma == 0.0 ? 0 : d/ma;

  return length(r) == 1 ? (array_map(Double_Type, &map_max, ma, mi, r, g, b, d)[0], s[0], v[0]) :
    (array_map(Double_Type, &map_max, ma, mi, r, g, b, d), s, v);
}
 