define get_color_map (name)
%!%+
%\function{get_color_map}
%\synopsis{Get an intrinsic color map or construct one}
%\usage{UInt_Type[256] map = get_color_map(name);}
%\qualifiers{
%  \qualifier{reverse}{Reverse returned map}
%}
%\description
%  Retreive a colormap, either from the internal list, or
%  constructed from a color palette.
%
%  Qualifiers are passed to the constructor.
%
%\seealso{get_color_palette, png_get_colormap_names}
%!%-
{
  variable map = NULL;

#ifexists png_add_colormap
  ifnot (qualifier_exists("palette"))
  {
    try
      map = png_get_colormap(name);
    catch AnyError;
  }

  if (NULL != map)
    return map;
#endif

  try
    map = get_color_palette(name, 256;; __qualifiers);
  catch AnyError:
    throw UsageError, "Unknown palette name '$name'";

  if (length(map)<256)
    map = get_color_palette("blend", 256; colors=map);

  if (qualifier_exists("reverse"))
    map = reverse(map);

  if (_typeof(map) == String_Type)
    return rgb2hex(hex2rgb(map));
  return map;
}
