define rgb2hex (r, g, b)
%!%+
%\function{rgb2hex}
%\synopsis{converts r, g, b values to 24 bit color values.}
%\usage{Integer_Type rgb2hex(Double/Int_Type r, g, b)}
%\qualifiers{
%  \qualifier{string}{returns value as hex encoded string.}
%}
%\description
%  Convert given color values to one 24 bit value.
%  If values are doubles they are interpreted as ranging
%  from 0 to 1. If integers they are interpreted as
%  8 bit (0-255) color values.
%\seealso{rgb2hex}
%!%-
{
  if (_typeof(r) == Double_Type) r *= 255;
  if (_typeof(g) == Double_Type) g *= 255;
  if (_typeof(b) == Double_Type) b *= 255;

  variable col = (int(_max(0, _min(r, 255)))<<16)
    + (int(_max(0, _min(g, 255)))<<8)
      + (int(_max(0, _min(b, 255))));

  if (qualifier_exists("str") or qualifier_exists("string"))
    col = array_map (String_Type, &sprintf, "%06x", col) ;
  else if (qualifier_exists("Str") or qualifier_exists("String")
	   or qualifier_exists("STR") or qualifier_exists("STRING"))
    col = array_map (String_Type, &sprintf, "%06X", col) ;
  

  return length(col)==1 ? col[0] : col;
}

private define map_hexstr (str)
{
  variable r, g, b;
  () = sscanf(str, "%02x%02x%02x", &r, &g, &b);
  return r, g, b;
}

define hex2rgb (col)
%!%+
%\function{hex2rgb}
%\synopsis{converts a 24 bit color value to r, g, b, values.}
%\usage{Integer_Type r, g, b hex2rgb(Int/String_Type col)}
%\qualifiers{
%  \qualifier{float}{If given, the channels are instead returned in the range (0-1).}
%}
%\description
%  Converts a hex color to RGB values. Can be either 24 bit
%  value or string.
%\seealso{rgb2hex}
%!%-
{
  variable r, g, b;
  if (_typeof(col) == String_Type)
    (r, g, b) = array_map(UInt_Type, UInt_Type, UInt_Type, &map_hexstr, strlow(col));
  else
    (r, g, b) = (col & (0xff0000))>>16, (col & 0xff00)>>8, col & 0xff;

  if (qualifier_exists("float"))
    (r, g, b) = (r/255., g/255., b/255.);

  return length(r) == 1 ? (r[0], g[0], b[0]) : (r, g, b);
}
