
define orbitalphase()
%!%+
%\function{orbitalphase}
%\synopsis{calculates an orbital phase at a given MJD time from an ephemeris}
%\usage{Double_Type phi = orbitalphase(Double_Type MJD, T0, P0, [Pdot, [Pddot ]]);
%\altusage{Double_Type phi = orbitalphase(Double_Type MJD, String_Type ephemeris);}
%}
%\description
%    \code{phi = (MJD-T0)/P - floor( (MJD-T0)/P );}
%    where P = P0 + ( Pdot/2. + Pddot*MJD/6. )*MJD ;
%    The string \code{ephemeris} can be used to refer to an internally
%    stored (T0, P) pair or, if available, (T0, P, Pdot, Pddot) quadruple.
%
%    If \code{orbitalphase} is called with an unknown \code{ephemeris},
%    the allowed values are shown by \code{get_ephemeris}.
%\qualifiers{
%\qualifier{numpulse}{return pulse number instead of phase}
%}
%\seealso{get_ephemeris}
%!%-
{
  variable MJD, T0, P, ephemeris=NULL, Pdot=0, Pddot=0;
  switch(_NARGS)
  { case 2: (MJD, ephemeris) = ();
      try { variable ephem = get_ephemeris(ephemeris); }
      catch AnyError: { return; }
      T0 = ephem.T0 ;
      P = ephem.P ;
      if(struct_field_exists(ephem, "Pdot"))   Pdot=ephem.Pdot ;
      if(struct_field_exists(ephem, "Pddot"))  Pddot=ephem.Pddot ;
  }
  { case 3: (MJD, T0, P) = ();              }
  { case 4: (MJD, T0, P, Pdot) = ();        }
  { case 5: (MJD, T0, P, Pdot, Pddot) = (); }
  { help(_function_name()); return; }

  MJD -= 1.*T0;
  variable phi = MJD/( P + (Pdot/2. + Pddot*MJD/6.)*MJD );
  if(qualifier_exists("numpulse"))  return phi;
  return phi-floor(phi);
}
